﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Core;
using HeuristicLab.Parameters;
using HeuristicLab.Common;
using HeuristicLab.Data;

namespace HeuristicLab.Problems.MetaOptimization {
  [StorableClass]
  public class AverageDoubleValueCrossover : SingleSuccessorOperator, IDoubleValueCrossover, IStochasticOperator {
    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }

    public AverageDoubleValueCrossover() { }
    [StorableConstructor]
    protected AverageDoubleValueCrossover(bool deserializing) : base(deserializing) { }
    protected AverageDoubleValueCrossover(AverageDoubleValueCrossover original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new AverageDoubleValueCrossover(this, cloner);
    }

    public void Apply(IRandom random, DoubleValue value, DoubleValue other, DoubleValueRange range) {
      ApplyStatic(random, value, other, range);
    }

    public static void ApplyStatic(IRandom random, DoubleValue value, DoubleValue other, DoubleValueRange range) {
      value.Value = (value.Value + other.Value) / 2;
      value.Value = range.ApplyStepSize(value.Value);
    }
  }
}
